{% for toc_item in page.toc %}
    <li class="toctree-l{{ navlevel + 1 }}"><a href="{{ toc_item.url }}">{{ toc_item.title }}</a></li>
    {% if toc_item.children %}
        <ul>
        {% for toc_item in toc_item.children %}
            <li><a class="toctree-l{{ navlevel + 2 }}" href="{{ toc_item.url }}">{{ toc_item.title }}</a></li>
        {% endfor %}
        </ul>
    {% endif %}
{% endfor %}
